// ==============================================================
//                 ORBITER MODULE: Orion
//                  Part of the ORBITER SDK
//          Copyright (C) 2001-2003 Martin Schweiger
//                   All rights reserved
//
// SoyuzESC.cpp
// Reference implementation of Atlantis Tank vessel class module
// Note: This module takes control of the tank after separation
// from the orbiter.
// ==============================================================

#define ORBITER_MODULE
#include "soyuzdes.h"
#include "math.h"

// ==============================================================
// Specialised vessel class CEVIS
// ==============================================================

// Constructor
SoyuzESC::SoyuzESC (OBJHANDLE hObj)
: VESSEL2(hObj)
{
	// preload mesh
	hSoyuzESCmesh = oapiLoadMeshGlobal ("SoyuzESC");
}

// ==============================================================
// Callback functions
// ==============================================================

// Set Tank class specs
void SoyuzESC::clbkSetClassCaps (FILEHANDLE cfg)
{

PARTICLESTREAMSPEC srb_contrail = {
	0, 12.0, 3, 150.0, 0.4, 8.0, 4, 3.0, PARTICLESTREAMSPEC::DIFFUSE,
	PARTICLESTREAMSPEC::LVL_PSQRT, 0, 0.5,
	PARTICLESTREAMSPEC::ATM_PLOG, 1e-6, 0.1
};
PARTICLESTREAMSPEC srb_exhaust = {
	0, 3.0, 40, 250.0, 0.04, 0.7, 10, 3.0, PARTICLESTREAMSPEC::EMISSIVE,
	PARTICLESTREAMSPEC::LVL_FLAT, 1, 1,
	PARTICLESTREAMSPEC::ATM_FLAT, 1, 1
};

PARTICLESTREAMSPEC escape_exhaust = {
	0, 1.5, 1000.0, 100.0, 0.01, 0.5, 5, 0.1, PARTICLESTREAMSPEC::EMISSIVE,
	PARTICLESTREAMSPEC::LVL_PSQRT, 0, 0.5,
	PARTICLESTREAMSPEC::ATM_PLOG, 0.0, 0.0
};

	escape_exhaust.tex = oapiRegisterParticleTexture ("Contrail");

	SetEnableFocus (false);
	// Tank cannot receive input focus

	SetSize (7.0);
	SetEmptyMass (2950.0);

	// Note that the Tank instance is only created after separation from
	// the orbiter, so the actual fuel mass will always be much smaller

	// Tank has no engines of its own

	SetCW (0.2, 0.3, 1.2, 1.2);
	SetCrossSections (_V(2.65, 2.75, 1.85));
	VECTOR3 rd = {0.7,0.7,0.3};
	SetRotDrag (rd);
	SetPMI (_V(1.8, 1.8, 0.25));
	SetPitchMomentScale (1e-4);
	SetBankMomentScale (1e-4);

	VECTOR3 co = {0,0,0};
	SetCameraOffset (co);
	// Note that the camera offset should not be required
	// since the Tank doesn't define a 'cockpit'

	SetCOG_elev (0.0);
	SetTouchdownPoints (_V(-2.70, -2.70, -0.9), _V(2.70, 0.0, -0.9), _V(2.70, -2.70, -0.9));
	SetLiftCoeffFunc (0);

	AddMesh (hSoyuzESCmesh);
	
	ph_escape = CreatePropellantResource( ER_FUEL );
	th_escape = CreateThruster( _V( 0,0,-1 ), _V(0,0,1), 100000.0, ph_escape, 250.0);
	SURFHANDLE tex = oapiRegisterExhaustTexture ("Exhaust2");
	srb_exhaust.tex = oapiRegisterParticleTexture ("Contrail2");
	AddExhaust (th_escape, 6.0, .42, _V(0,0.589,2.243), _V(0,.666,-1), tex);
	AddExhaust (th_escape, 6.0, .42, _V(0,-0.589,2.243), _V(0,-.666,-1), tex);
	AddExhaust (th_escape, 6.0, .42, _V(0.589,0,2.243), _V(.666,0,-1), tex);
	AddExhaust (th_escape, 6.0, .42, _V(-0.589,0,2.243), _V(-.666,0,-1), tex);
	AddExhaustStream (th_escape, _V(0,2.03,0.00), &srb_contrail);
	AddExhaustStream (th_escape, _V(0,4.25,-3.75), &srb_exhaust);
	AddExhaustStream (th_escape, _V(0,-2.03,0.00), &srb_contrail);
	AddExhaustStream (th_escape, _V(0,-4.25,-3.75), &srb_exhaust);
	AddExhaustStream (th_escape, _V(2.03,0,0.00), &srb_contrail);
	AddExhaustStream (th_escape, _V(4.25,0,-3.75), &srb_exhaust);
	AddExhaustStream (th_escape, _V(-2.03,0,0.00), &srb_contrail);
	AddExhaustStream (th_escape, _V(-4.25,0,-3.75), &srb_exhaust);
	thg_ER = CreateThrusterGroup (&th_escape, 1, THGROUP_MAIN);

}

void SoyuzESC::clbkPostCreation ()
{
	SetPropellantMass (ph_escape, 1200.0);
	SetThrusterGroupLevel (thg_ER, 1.0);
}
	// Simulation time step
void SoyuzESC::clbkPostStep (double simt, double simdt, double mjd)
{
	if (GetAltitude() < 5.0) oapiDeleteVessel (GetHandle());
	SetThrusterGroupLevel (thg_ER, 1.0);
}

// ==============================================================
// API interface
// ==============================================================

// Initialisation
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new SoyuzESC (hvessel);
}

// Cleanup
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (SoyuzESC*)vessel;
}

// Vessel status initialisation
DLLCLBK void ovcSetState (VESSEL *vessel, const VESSELSTATUS *status)
{
	vessel->DefSetState (status);	        // default initialisation
}
