// ==============================================================
// $Id: SoyuzDES.h,v 1.7 2006/03/18 21:06:20 dh219 Exp $
// Soyuz TMA by D Henderson orbiter@aibs.org.uk
// Copyright 2004
// Released under the terms of the LGPL: http://www.gnu.org/licenses/lgpl.txt

/*
    This software and sourcecode is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

// ==============================================================

#ifndef __SoyuzDES_H
#define __SoyuzDES_H

#include "orbitersdk.h"
#include <stdio.h>
#include <math.h>

// ==========================================================
// Some Soyuz TMA-related parameters
// ==========================================================

const double MASS = 6320.0;

const double FUEL = 900.0;
const double ER_FUEL = 10.0;
const double FUELRCS = 45.0;
const double FUELSRM = 70.0;

// ==========================================================
// Some Soyuz Re-Entry Module-related parameters
// ==========================================================

const double ENTRY_MAX_PROPELLANT_MASS = 0;
// Main Engine propellant mass [kg]

const double MASSENTRY = 2950.0;
const double HS_EMPTY_MASS = 500;
const double R7TMAER_MASS = 500;
// Main Engine empty mass

const double ENTRY_ISP0 = 5000;
const double ENTRY_ISP1 = 4000;
// Main Engine vacuum and sea-level fuel-specific impulse [m/s]

const double ENTRY_THRUST = 0;
// Vacuum Main Engine thrust per unit [N]

const VECTOR3 ENTRY_CS = {4.25,4.19,3.87};
// Orbiter cross sections (projections into principal axes) [m^2]

const VECTOR3 ENTRY_PMI = {0.78,0.79,0.63};
// Orbiter Inertia Tensor

const double ENTRY_CW[4] = {5.5, 0.1, 3.4, 3.4};
// Orbiter wind resistance coefficients in principal directions (+z, -z, +-x, +-y)

const VECTOR3 ENTRY_RD = {0.07,0.07,0.003};
// Orbiter Rotational Drag (projections into principal axes) [m^2]

// ==========================================================
// Some Generic parameters
// ==========================================================

const UINT MESH_UNDEFINED = (UINT)-1;
const VECTOR3 REENTRY_CS = { 3.87, 4.25, 4.19 };
const double REENTRY_CW[4] = { 5.5, 0.1, 3.4, 3.4 };
const VECTOR3 REENTRY_RD = {0.07,0.07,0.003};

const double ABORT_OPERATING_SPEED = 10;

// ==========================================================
// Begining of Class Definitions
// ==========================================================

class SoyuzDES: public VESSEL2 {
public:

	enum DoorStatus { DOOR_CLOSED, DOOR_OPEN, DOOR_CLOSING, DOOR_OPENING } abort_status;
	SoyuzDES(OBJHANDLE hObj, int fmodel);
	~SoyuzDES ();

	int status;
	bool bManualSeparate;
	int shieldstatus;

	double abort_proc; // abort latch deployment state (0=retracted, 1=deployed)

	// Overloaded callback functions
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkSetStateEx (const void *status);
	void clbkLoadStateEx (FILEHANDLE scn, void *vs);
	void clbkSaveState (FILEHANDLE scn);
	void clbkFocusChanged (bool getfocus, OBJHANDLE hNewVessel, OBJHANDLE hOldVessel);
	void clbkPostStep (double simt, double simdt, double mjd);
	bool clbkPlaybackEvent (double simt, double event_t, const char *event_type, const char *event);
	int  clbkConsumeBufferedKey (DWORD key, bool down, char *kstate);
	void clbkVisualCreated (VISHANDLE vis, int refcount);
	void clbkVisualDestroyed (VISHANDLE vis, int refcount);
	void clbkAnimate (double simt);
	bool clbkLoadVC (int id);

	MESHHANDLE htma;
	MESHHANDLE hentry;
	MESHHANDLE hpara;
	MESHHANDLE hdrogue;
	MESHHANDLE hHSmesh;

	VISHANDLE vis;      // handle for visual - note: we assume that only one visual per object is created!

	DOCKHANDLE dh;


private:

	void SetMainStage ();
	void SetEntryStage ();
	void SetParaStage ();
	void SetDrogueStage();
	void SetRetroStage();
	void SetGroundedStage();
	void CalcParaAngles();
	void DefineAnimations();
	void OperateAbort (DoorStatus action);
	void RevertAbort ();
	void AddThrusters();
	void ClearMeshes();
	void UpdateMesh();
	void BlowOrbiter();
	void BlowHeatShield();
	void AddDescVisual ( const VECTOR3 &ofs );
	void AddHSVisual ( const VECTOR3 &ofs );
	void AddTMAVisual ( const VECTOR3 &ofs );
	void AddParaVisual ( const VECTOR3 &ofs );
	void AddDrogueVisual ( const VECTOR3 &ofs );

	PROPELLANT_HANDLE ph_first;	
	PROPELLANT_HANDLE ph_second;
	PROPELLANT_HANDLE ph_srm;
	THRUSTER_HANDLE th_first;
	THRUSTER_HANDLE th_pup[4];
	THRUSTER_HANDLE th_pdown[4];
	THRUSTER_HANDLE th_yleft[4];
	THRUSTER_HANDLE th_yright[4];
	THRUSTER_HANDLE th_rleft[4];
	THRUSTER_HANDLE th_rright[4];
	THRUSTER_HANDLE th_up[4];
	THRUSTER_HANDLE th_down[4];
	THRUSTER_HANDLE th_left[4];
	THRUSTER_HANDLE th_right[4];
	THRUSTER_HANDLE th_fore[4];
	THRUSTER_HANDLE th_aft[4];
	THRUSTER_HANDLE th_srm;

	SURFHANDLE tex;

	VECTOR3 VZERO;

	int targetstatus;

	VECTOR3 comshift;
	VECTOR3 OFS_DES;
	VECTOR3 OFS_HS;
	
	UINT anim1, anim2, anim3;
	UINT mesh_tma, mesh_para, mesh_drogue, mesh_desc, mesh_HS;

	double i_state;

	VESSEL *chute;

	bool reenteractive;
	bool reentercoasting;

	double target;
	double chute_timer;
	double start_time;
	double start_alt;

	UINT refresh;
	UINT anim_abort;

	bool render_cockpit;
	VCHUDSPEC huds;
	VCMFDSPEC mfds_left;
	VCMFDSPEC mfds_right;
};

class SoyuzESC: public VESSEL2 {
public:
	SoyuzESC (OBJHANDLE hObj);
	// Construct interface from existing object

	// Overloaded callback functions
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkPostCreation ();
	void clbkPostStep (double simt, double simdt, double mjd);

private:
	MESHHANDLE hSoyuzESCmesh;
	UINT mesh_SoyuzESC;                              // index for Inter-Stage mesh
	PROPELLANT_HANDLE ph_escape;
	THRUSTER_HANDLE th_escape;
	THGROUP_HANDLE thg_ER;                   // handles for thruster groups
};

#endif