// ==============================================================
//                 ORBITER MODULE: SoyPad
//                  Part of the ORBITER SDK
//          Copyright (C) 2001-2003 Martin Schweiger
//                   All rights reserved
//
// SoyPad.cpp
// Reference implementation of Soyuz Launch Pad
// class module
//
// ==============================================================

#define STRICT
#define ORBITER_MODULE
#include "SoyuzPad.h"
#include <stdio.h>
#include <fstream>

// ==============================================================
// Global (class-wide) parameters

struct {
	HINSTANCE hDLL;
} g_Param;

// ==============================================================
// Local prototypes


// ==============================================================
// Specialised vessel class AResV
// ==============================================================

// --------------------------------------------------------------
// Constructor
// --------------------------------------------------------------
SoyuzPad::SoyuzPad (OBJHANDLE hObj, int fmodel)
: VESSEL2 (hObj, fmodel)
{
	status          = 0;
	pad_status     = DOOR_CLOSED;
	pad_proc       = 0.0;

	mesh_Pad           = MESH_UNDEFINED;

	vis             = NULL;
	
	render_cockpit    = false;
	mfds_left.ngroup  = 125;
	mfds_right.ngroup = 126;
	huds.ngroup       = 127;
	huds.size         = 0.252;

	// propellant resources

	// preload meshes
	hPadmesh    = oapiLoadMeshGlobal ("SoyuzPad/SoyuzPad2");

	DefineAnimations();
	bManualSeparate = false;
	vis             = NULL;

	}

SoyuzPad::~SoyuzPad()
{

//	delete th_ASI_main;

}
// Update moving parts of the orbiter's visual: payload bay doors and gear
// This should only be called when the visual exists, e.g. from within
// clbkVisualCreated or clbkAnimate

void SoyuzPad::AddPadVisual (const VECTOR3 &ofs)
{
	huds.hudcnt = _V(ofs.x - 0.5479, ofs.y + 2.325, ofs.z + 15.777);

	if (mesh_Pad == MESH_UNDEFINED) {

		// ***** Load meshes

//		mesh_Pad = AddMesh (hCAPmesh, &ofs);
		mesh_Pad = InsertMesh (hPadmesh, 0, &ofs);
		SetMeshVisibilityMode (mesh_Pad, MESHVIS_ALWAYS);
		mfds_left.nmesh = mfds_right.nmesh = huds.nmesh = mesh_Pad;

		// ***** Docking definitions


		// ***** Attachment definitions

		// ***** Cockpit camera definition

//		SetCameraOffset (_V(ofs.x-0.55,ofs.y+2.4,ofs.z+15.0));
		oapiVCRegisterHUD (&huds); // register changes in HUD parameters
	}
}

void SoyuzPad::UpdateMesh ()
{
	// update animation states
	SetAnimation (anim_pad, pad_proc);

}

void SoyuzPad::ClearMeshes ()
{
	VESSEL::ClearMeshes();
	mesh_Pad      = MESH_UNDEFINED;
}

void SoyuzPad::OperatePad (DoorStatus action)
{

	pad_status = action;
	RecordEvent ("LAUNCHPAD", action == DOOR_CLOSING ? "Stored" : "Deployed");
}

void SoyuzPad::RevertPad ()
{

	OperatePad (pad_status == DOOR_CLOSED || pad_status == DOOR_CLOSING ?
		DOOR_OPENING : DOOR_CLOSING);
}

// --------------------------------------------------------------
// Define animation sequences for moving parts
// --------------------------------------------------------------
void SoyuzPad::DefineAnimations (void)
{
	// ***** 1. Solar Panel animation *****

	// 0.5 radians = 90 degrees - 0.0056 radians = 1 degree

	// 0.1111 radians = 20 degrees

	static UINT RightGrp[42] = {146,147,148,149,150,151,152,153,154,155,156,157,158,159,160,161,162,163,164,165,166,167,168,169,170,171,172,173,174,175,176,177,178,179,180,181,182,183,184,185,274,275};
	static MGROUP_ROTATE RightPanel (0, RightGrp, 42,
		_V(6.586,1.245,6.472), _V(1,0,-1), 40*RAD);
	static UINT FrontGrp[43] = {105,106,107,108,109,110,111,112,113,114,115,116,117,118,119,120,121,122,123,124,125,126,127,128,129,130,131,132,133,134,135,136,137,138,139,140,141,142,143,144,145,272,273};
	static MGROUP_ROTATE FrontPanel (0, FrontGrp, 43,
		_V(-5.586,1.245,6.472), _V(-1,0,-1), -40*RAD);
	static UINT LeftGrp[43] = {64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95,96,97,98,99,100,101,102,103,104,270,271};
	static MGROUP_ROTATE LeftPanel (0, LeftGrp, 43,
		_V(-5.586,1.245,-5.730), _V(-1,0,1), 40*RAD);
	static UINT RearGrp[44] = {20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63};
	static MGROUP_ROTATE RearPanel (0, RearGrp, 44,
		_V(6.586,1.245,-5.730), _V(1,0,1), -40*RAD);
	static UINT LateralGrp1[16] = {276,277,278,279,280,281,282,283,284,285,286,287,288,0,2,313};
	static MGROUP_ROTATE Lateral1 (0, LateralGrp1, 16,
		_V(7.346,1.073,2.622), _V(0,0,-1), 90*RAD);
	static UINT LateralGrp2[16] = {289,290,291,292,293,294,295,296,297,298,299,300,301,1,3,314};
	static MGROUP_ROTATE Lateral2 (0, LateralGrp2, 16,
		_V(-8.313,1.038,2.622), _V(0,0,1), 90*RAD);

	anim_pad = CreateAnimation (0);

	AddAnimationComponent (anim_pad, 0.20, 0.45, &Lateral1);
	AddAnimationComponent (anim_pad, 0.20, 0.45, &Lateral2);
	AddAnimationComponent (anim_pad, 0.95, 1.00, &RightPanel);
	AddAnimationComponent (anim_pad, 0.95, 1.00, &LeftPanel);
	AddAnimationComponent (anim_pad, 0.95, 1.00, &FrontPanel);
	AddAnimationComponent (anim_pad, 0.95, 1.00, &RearPanel);


}
// ==============================================================
// Overloaded callback functions
// ==============================================================

// --------------------------------------------------------------
// Set vessel class capabilities from config file
// --------------------------------------------------------------
void SoyuzPad::clbkSetClassCaps (FILEHANDLE cfg)
{
	if (!oapiReadItem_bool (cfg, "RenderCockpit", render_cockpit))
		render_cockpit = false;
}
// --------------------------------------------------------------
// Set status from a VESSELSTATUS2 structure
// --------------------------------------------------------------
void SoyuzPad::clbkSetStateEx (const void *status)
{
	// default parameter initialisation
	DefSetStateEx (status);

	// reset vessel-specific parameters to defaults
	//status = 5;
}

// --------------------------------------------------------------
// Read status from scenario file
// --------------------------------------------------------------
void SoyuzPad::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
    char *line;
//	double met = 0.0; // mission elapsed time
	bManualSeparate = false;

	while (oapiReadScenario_nextline (scn, line)) {
        if (!strnicmp (line, "CONFIGURATION", 13)) {
            sscanf (line+13, "%d", &status);
		} else if (!strnicmp (line, "MET", 3)) {
			sscanf (line+3, "%lf", &met);
		} else if (!strnicmp (line, "LAUNCHPAD", 9)) {
			sscanf (line+9, "%d%lf", &pad_status, &pad_proc);
		} else { 
            ParseScenarioLineEx (line, vs);
			// unrecognised option - pass to Orbiter's generic parser
        }
    }

	ClearMeshes();
	switch (status) {
	case 0:
		break;
	case 1:
		break;
	case 2:
		break;
	case 3:
		break;
	case 4:
		break;
	case 5:
		break;
	case 6:
	case 7:
		break;
	}
	AddPadVisual (_V(0,0,0));
	UpdateMesh ();
}

// --------------------------------------------------------------
// Write status to scenario file
// --------------------------------------------------------------
void SoyuzPad::clbkSaveState (FILEHANDLE scn)
{
	char cbuf[256];

	// default vessel parameters
	VESSEL2::clbkSaveState (scn);

	// custom parameters
	oapiWriteScenario_int (scn, "CONFIGURATION", status);

	sprintf (cbuf, "%d %0.4f", pad_status, pad_proc);
	oapiWriteScenario_string (scn, "Solar Panel", cbuf);

}

// --------------------------------------------------------------
// Vessel gains or loses input focus
// --------------------------------------------------------------
void SoyuzPad::clbkFocusChanged (bool getfocus, OBJHANDLE newv, OBJHANDLE oldv)
{
	if (getfocus) {
		oapiDisableMFDMode (MFD_LANDING);
		// no VTOL MFD mode for Atlantis
	}
	if(!getfocus)
	{
		strcpy(oapiDebugString(),"");
	}
}

// --------------------------------------------------------------
// Simulation time step
// --------------------------------------------------------------
void SoyuzPad::clbkPostStep (double simt, double simdt, double mjd)
{

	if (pad_status >= DOOR_CLOSING) {
		double da = simdt * PAD_OPERATING_SPEED;
		if (pad_status == DOOR_CLOSING) { // retract gear
			if (pad_proc > 0.0) pad_proc = max (0.0, pad_proc-da);
			else  {
				pad_status = DOOR_CLOSED;
			}
		} else {                           // deploy gear
			if (pad_proc < 1.0) {
				pad_proc = min (1.0, pad_proc+da);  //Door Opening
			}
			else  {
				pad_status = DOOR_OPEN;
			}
		}
		SetAnimation (anim_pad, pad_proc);
	}

}
// --------------------------------------------------------------
// Respond to playback event
// --------------------------------------------------------------
bool SoyuzPad::clbkPlaybackEvent (double simt, double event_t, const char *event_type, const char *event)
{
	if (_stricmp (event_type, "JET")) {
			return true;
	} else if (_stricmp (event_type, "LAUNCHPAD")) {
		OperatePad (_stricmp (event, "CLOSE") ? DOOR_CLOSING : DOOR_OPENING);
		return true;
	}

	return false;
}

// --------------------------------------------------------------
// Atlantis mesh loaded
// --------------------------------------------------------------
void SoyuzPad::clbkVisualCreated (VISHANDLE _vis, int refcount)
{
	if (refcount > 1) return; // we don't support more than one visual per object
	vis = _vis;

	UpdateMesh ();
}

// --------------------------------------------------------------
// Atlantis mesh discarded
// --------------------------------------------------------------
void SoyuzPad::clbkVisualDestroyed (VISHANDLE _vis, int refcount)
{
	if (vis == _vis) vis = NULL;
}

// --------------------------------------------------------------
// Update mesh animation state
// --------------------------------------------------------------
void SoyuzPad::clbkAnimate (double simt)
{
	UpdateMesh ();
}

// --------------------------------------------------------------
// Load virtual cockpit mode
// --------------------------------------------------------------
bool SoyuzPad::clbkLoadVC (int id)
{
	switch (id) {
	case 0:
		oapiVCRegisterHUD (&huds);
		oapiVCRegisterMFD (MFD_LEFT, &mfds_left);
		oapiVCRegisterMFD (MFD_RIGHT, &mfds_right);
		return true;
	default:
		return false;
	}
}

// --------------------------------------------------------------
// Keyboard interface handler (buffered key events)
// --------------------------------------------------------------
int SoyuzPad::clbkConsumeBufferedKey (DWORD key, bool down, char *kstate)
{
	if (!down) return 0; // only process keydown events


	switch (key) {
	case OAPI_KEY_L:  // "Jettison"
		if (!Playback()) {
			bManualSeparate = true;
		}
		return 1;
	case OAPI_KEY_K:  // "Solar Panels"
		if (!Playback()) RevertPad ();
		return 1;
	}

	return 0;
}

// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Module initialisation
// --------------------------------------------------------------
DLLCLBK void InitModule (HINSTANCE hModule)
{
	g_Param.hDLL = hModule;
}

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new SoyuzPad (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (SoyuzPad*)vessel;
}
