// ==============================================================
//                 ORBITER MODULE: Atlantis
//                  Part of the ORBITER SDK
//          Copyright (C) 2001-2003 Martin Schweiger
//                   All rights reserved
//
// Orion.h
// Class interface of Atlantis (Space Shuttle) vessel class
// module and associated subclasses (SRB, tank)
// ==============================================================

#ifndef __SoyuzPad_H
#define __SoyuzPad_H

#include "orbitersdk.h"
#include <math.h>

const double PAD_OPERATING_SPEED = 0.1;
// Deployment speed of the Ku Band antenna (1/sec)
// cycle is 20 sec


// ==========================================================
// Mesh group indices for some components
// ==========================================================

const int ORBITER_MESHGROUP_PANEL = 96;
const UINT MESH_UNDEFINED = (UINT)-1;

// ==========================================================
// Interface for derived vessel class: EDS
// ==========================================================

class SoyuzPad: public VESSEL2 {

public:
	enum DoorStatus { DOOR_CLOSED, DOOR_OPEN, DOOR_CLOSING, DOOR_OPENING } pad_status;
	SoyuzPad (OBJHANDLE hObj, int fmodel);
	~SoyuzPad ();

	int status; // 0=launch configuration
	            // 1=SRB engaged
	            // 2=SRB separated
	            // 3=Escape Tower separated
	            // 4=AresI separated
	            // 5=IS separated
	            // 6=SM separated
	            // 7=Capsule separated
	bool bManualSeparate; // flag for user-induced booster or tank separation

	double met; // mission elapsed time

	double pad_proc; // landing gear deployment state (0=retracted, 1=deployed)
	VISHANDLE vis;      // handle for visual - note: we assume that only one visual per object is created!
	MESHHANDLE hPadmesh; // mesh handles

	// Overloaded callback functions
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkSetStateEx (const void *status);
	void clbkLoadStateEx (FILEHANDLE scn, void *vs);
	void clbkSaveState (FILEHANDLE scn);
	void clbkFocusChanged (bool getfocus, OBJHANDLE hNewVessel, OBJHANDLE hOldVessel);
	void clbkPostStep (double simt, double simdt, double mjd);
	bool clbkPlaybackEvent (double simt, double event_t, const char *event_type, const char *event);
	int  clbkConsumeBufferedKey (DWORD key, bool down, char *kstate);
	void clbkVisualCreated (VISHANDLE vis, int refcount);
	void clbkVisualDestroyed (VISHANDLE vis, int refcount);
	void clbkAnimate (double simt);
	bool clbkLoadVC (int id);

private:

	void DefineAnimations (void);
	void AddPadVisual (const VECTOR3 &ofs);
	void Jettison ();
	void UpdateMesh ();
	void ClearMeshes ();
	void OperatePad (DoorStatus action);
	void RevertPad ();

	UINT anim_pad;                              // handle for landing gear animation
	UINT mesh_Pad;                             // index for EDS mesh

	bool render_cockpit;
	VCHUDSPEC huds;
	VCMFDSPEC mfds_left;
	VCMFDSPEC mfds_right;
};

#endif // !__Orion_H
